% AGGDESCRIPTIVES Aggregates yearly data obtained by using getdescriptivestats. 
% Outputs histograms.
% Set the input argument flag true if you want to normalize distributions 
% (or leave empty).
% OUTPUTS:
% aggdescriptives{1}=degree distribution of people 
% (degree=number of board positions)
% aggdescriptives{2}=histogram of distances of new individuals, 
% note the last value is the number of persons not found in any neighborhood,
% aggdescriptives{3}=degree histogram of individuals who have lost a position,
% aggdescriptives{4}=degree histogram of new individuals conditional on that 
% they are found in some neighborhood,
% aggdescriptives{5}=median number of positions in each firm,
% aggdescriptives{6}=ndeg; is the degree distribution of all positions,
% aggdescriptives{7}=ndegold; is the degree distribution of opened vacancies,
% aggdescriptives{8} mean clustering coefficient,
% aggdescriptives{9} probability that a newly recruited board member is an 
% outsider, i.e., not found from the boards of the previous year,
% aggdescriptives{10} probabilities that positions with different 
% degrees are opened,
% aggdescriptives{11} probability that a vacancy opens
% REQUIREMENTS: add2dist

function aggdescriptives=aggregatedist(descriptives,flag)

if nargin<2
flag=true;
end

n=size(descriptives{2,1},2);
aggdescriptives{1}=[];
aggdescriptivestemp{2}=zeros(1,n);
aggdescriptives{3}=[];
aggdescriptives{4}=[];
aggdescriptives{5}=median(cell2mat(descriptives(5,:)),2);%median number of seats in each firm
aggdescriptives{6}=[];
aggdescriptives{7}=[];
aggdescriptives{8}=mean(cell2mat(descriptives(8,:)));%mean clustering coefficient

nvac=0;
nopen=0;

T=size(descriptives,2);

for i=1:T-1
aggdescriptives{1}=add2dist(descriptives{1,i},aggdescriptives{1},1);
aggdescriptivestemp{2}=aggdescriptivestemp{2}+descriptives{2,i};
aggdescriptives{3}=add2dist(descriptives{3,i},aggdescriptives{3},1);
aggdescriptives{4}=add2dist(descriptives{4,i},aggdescriptives{4},1);
aggdescriptives{6}=add2dist(descriptives{6,i},aggdescriptives{6},1);
aggdescriptives{7}=add2dist(descriptives{7,i},aggdescriptives{7},1);
%these are required to get a probability that a vacancy opens
nvac=nvac+sum(descriptives{6,i});%+sum(descriptives{9,i}); %number of positions
nopen=nopen+sum(descriptives{7,i});
end
aggdescriptives{1}=add2dist(descriptives{1,end},aggdescriptives{1},1);
aggdescriptives{1}(1)=aggdescriptives{1}(1)+sum(sum(cell2mat(descriptives(9,:))));%add empty seats to degree one
aggdescriptives{2}=aggdescriptivestemp{2};
if nargin==1 || flag==true;
aggdescriptives{1}=aggdescriptives{1}./T;
aggdescriptives{2}=aggdescriptives{2}./sum(aggdescriptives{2});
aggdescriptives{3}=aggdescriptives{3}./sum(aggdescriptives{3});
aggdescriptives{4}=aggdescriptives{4}./sum(aggdescriptives{4});
end
%sum(aggdescriptives{7}) is the total number of opened vacancies (including the ones left empty)
%number of positions left empty = sum(aggdescriptives{7})-descriptives{10,1}(2)
%total number of new comers = descriptives{10,1}(1)+sum(aggdescriptives{7})-descriptives{10,1}(2)
aggdescriptives{9}=descriptives{10,1}(1)/descriptives{10,1}(2); %probability that a new recruit is an outsider
%empirical probabilities that positions with different degrees are opened
for i=1:length(aggdescriptives{7})
aggdescriptives{10}(i)=aggdescriptives{7}(i)/aggdescriptives{6}(i);
end
%empirical probability that a vacancy opens
aggdescriptives{11}=nopen/nvac;

aggdescriptives{6}=add2dist(descriptives{6,end},aggdescriptives{6},1);
