% ELIST2ADJ Turn an edgelist into an adjacency matrix of a two mode network.
% INPUT: 
% yr = year of the edgelist that should be turned into an adjacency matrix. 
% Note the file containing the dgelist of year yr is assumed to be of the 
% form edgelistyr.tsv. 
% OUPUTS: 
% firms is a list of firms, 
% individuals is a list of individuals, 
% Adj is a (sparse) adjacency matrix with rows corresponding to firms and 
% columns individuals.
% Warning: this is rather slow.
function [firms,individuals,Adj]=elist2adj(yr)

r=[];c=[];d=[]; firms={}; individuals={};

year=num2str(yr);
curfile=strcat('edgelist',year,'.tsv');
fid1=fopen(curfile,'rt');

curline=fgetl(fid1);
temp=strsplit(curline,"\t");
firms{1}=temp{1,1};
individuals{1}=temp{1,2};
r=[r 1]; c=[c 1]; d=[d 1];

fcntr=2; icntr=2;
curline=fgetl(fid1);
while ischar(curline)

temp=strsplit(curline,"\t");

firmtrial=temp{1,1}; %take firm
idvtrial=temp{1,2}; %and individual
%test whether the individual is new, i.e., not already listed
[flag,ind]=ismember(idvtrial,individuals);
if ~strcmp(firmtrial,firms{fcntr-1}) %test whether the firm is new
firms{fcntr}=firmtrial;
if flag==false
r=[r fcntr]; c=[c icntr]; d=[d 1];
individuals{icntr}=idvtrial;
icntr=icntr+1;
else
r=[r fcntr]; c=[c ind]; d=[d 1];
end
fcntr=fcntr+1;%new firm, increase the counter
else %the same firm as before
if flag==false%new individual
r=[r fcntr-1]; c=[c icntr]; d=[d 1];
individuals{icntr}=idvtrial;
icntr=icntr+1;
else%individual who has already been listed
r=[r fcntr-1]; c=[c ind]; d=[d 1];
end
end %end of else-if
curline=fgetl(fid1);
end %end of WHILE
fclose(fid1);

Adj=sparse(r,c,d,fcntr-1,icntr-1);
Adj=spones(Adj);% just in case someone is listed twice in an edgelist
end %end of file
