% FINDINNEIGHBORHOODVAC This function computes the distance and degree 
% distributions of rewired edges.
% INPUTS: 
% firms IS A list of firms for which there are changes in their boards,
% A is an adjacency matrix of a two-mode network, i.e., m x n matrix where 
% m is the number of firms and n is the number of individuals,
% indic should be a logical matrix of the following form; indic(i,j) is true if
% individual j is new in the board of firm j,
% nsearch is the number of neghborhoods went through 
% (optional, if left empty the default is 45),
% Outputs: dist is the distance histogram of new people (except for the last element),
% degr is the degree histogram of new board members (if the same person is new to 
% multiple boards he/she is counted multiple times, i.e. each open position is 
% considered as a separate entity, so the distribution is for positions rather 
% than individuals)
% Note 1: if you want to know the number of opened vacancies new holder of which 
% is an outsider, take sum(dist)-sum(degr), sum(dist) is the total number of opened 
% vacancies and sum(degr) is the total number of people holding positions that are 
% found in the prior year population
% Note 2: the last element of dist contains people who are either not found or 
% who are new, so the people who are not found (in the main component) are dist(end)-(sum(dist)-sum(degr))
% REQUIREMENTS: fastsetdiff, add2dist
function [dist,degr]=findinneighborhoodvac(firms,indic,A,nsearch);

if nargin<4
nsearch=min(20,size(A,1));
end

B=spones(A*A');
nof=length(firms);
nfirm=size(A,1);
dist=zeros(1,nfirm);%initialization of the distance histogram
degr=[];%degree distribution of new individuals
%degr can take value zero when the new people is not in any firm
zerpeople=0;%people with zero degree, this is only for debugging
totalnew=0;%only for diagnostics, total number of new people

zerfirms=[];
zerfirmsind=[];

if nof>0

for j=1:nof
idvs{j}=find(A(firms(j),:)~=0);
newpeople{j}=find(indic(firms(j),:));%new people for firm j
ntosearch(j)=length(newpeople{j});
neighfirms{j}=[];
dst{j}=zeros(1,nfirm);
%truedist{j}=[];
%degr{j}=[];
if ~isempty(newpeople{j})
tempsum=full(sum(A(:,newpeople{j}))); %have to use full matrix, histc does not work for sparse
degr=add2dist(tempsum,degr,0,0); %note that the first bin contains people with no position, i.e., degree 0
%sum(tempsum)
if sum(tempsum)==0
ntosearch(j)=0;
zerfirms=[zerfirms firms(j)];
zerfirmsind=[zerfirmsind j];
dist(end)=dist(end)+length(newpeople{j});
end
end
end%end of for
%zerfirms

countaux=[1:1:nof];%this is an auxiliary list of firms to be went through
countaux=fastsetdiff(countaux,zerfirmsind); %do not go through those firms who have only new people
nofaux=length(countaux);
stopflag=false;
tempmat=B;
cntr=1;
cmax=min(nsearch,nfirm);%How many neighborhoods are searched

if isempty(countaux)
stopflag=true
end

while stopflag==false
remind=[];
for i=1:nofaux

tempfirms=fastsetdiff(find(tempmat(firms(countaux(i)),:)~=0),firms(countaux(i)));%firms that can be reached from countaux(i)

%neighboring firms
%people in neighboring firms
if ~isempty(neighfirms{countaux(i)}) && ~isempty(tempfirms)
tempfirms=fastsetdiff(tempfirms,neighfirms{countaux(i)});%find firms that cannot already be reached
end


if ~isempty(tempfirms)
neighfirms{countaux(i)}=sort([neighfirms{countaux(i)} tempfirms]);%create the list of neighboring firms (to keep track of the reachable firms)
trial=fastsetdiff(find(sum(A(tempfirms,:),1)),idvs{countaux(i)});%people in firms that can be reached in cntr steps and who are not already in the board of the firm

temp=newpeople{countaux(i)};
idx=lookup(temp,trial,'m');%this gives directly the intersection, so a test whether newpeople are found in trial
%Warning: older versions of octave do not accept option 'm', which means that this does not function in older versions!
%If you have an older version this solves the issue
%if ~isempty(idx) 
%tocheck=find(idx>0);%tho indices for which there might be a problem
%for k=1:length(tocheck)
%if trial(tocheck(k))!=temp(idx(tocheck(k)));
%idx(tocheck(k))=0;
%end
%end
%end

idx(idx==0)=[]; %idx is zero if not found, as a result we have indices of those people from temp who have been found

%update new people
if ~isempty(idx)
temp(idx)=[];%some of the people are found and need not be searched anymore
%length(idx)
dst{countaux(i)}(cntr)=length(idx);%how many people are found in the neighborhood
dist(cntr)=dist(cntr)+dst{countaux(i)}(cntr);
if ~isempty(temp)
newpeople{countaux(i)}=temp;%the rest should be searched
ntosearch(countaux(i))=length(temp);%number of people to be searched
else %if all people are found
remind=[remind i];
ntosearch(countaux(i))=0;%number of people to be searched
end%if isempty temp

end %if isempty(idx)
else
remind=[remind i]; %all firms that can be found in a neighborhood are went through
end%end of if isempty tempfirms
end%end of for

countaux(remind)=[];
if isempty(countaux)
stopflag=true;
else
nofaux=length(countaux);
end
cntr=cntr+1;
if cntr>=cmax || cntr>=nfirm
stopflag=true;
else
if rem(cntr,4)==0
tempmat=spones(B*tempmat);
else
tempmat=B*tempmat;
end
end

end%end of while
dist(end)=dist(end)+sum(ntosearch);%-sum(dist); %note the last value is the number of persons not found in any neighborhood

degr=degr(2:end);

%A consistency check
%sum(dist(1:end-1))
%sum(ntosearch)
%sum(dist)
%sum(degr)+zerpeople
%sum(degrr)
%totalnew
else %nof==0
dist=[];
degr=[];
end
