% FIRMSELECTION Firms that exists between years fyr and lyr and are selected 
% according to caseflag.
% caseflag=0; corresponds to taking the intersection of all giant components, 
% i.e., firms that have appear in the giant component each year 
% between fyr and lyr.
% caseflag=1; corresponds to taking the union of all giant components of 
% different years.
% caseflag=2; corresponds to taking all firms.
% caseflag=3; all firms that are indirectly or directly connected to 
% the main component (the recursive selection used in the manuscript).
% Set eflag=true if you do not care about the existence, i.e., include firms 
% even if they have disappeared from the data.
% REQUIREMENTS: elist2adj, firmsinpc
% and the edgelists should be stored in files edgelistyr.tsv (yr = year). 
function newlist=firmselection(fyr,lyr,firmlist,caseflag,eflag)

if nargin<=2
caseflag=[];
end

if nargin<=2 || isempty(firmlist)
firmlist=firmsinpc(fyr,lyr,caseflag);%Be prepared that executing firmsinpc takes a while. 
end

if nargin<5 || isempty(eflag) || eflag==false
newlist=firmlist;
for yr=fyr:lyr
[firms1,individuals1,A1]=elist2adj(yr);%make adjacency matrices from edgelists
newlist=intersect(firms1,newlist);%this line is to control that only firms that exist throughout the sample period are included
%Just a minor note: the data contains only firms that exist in 2013, but some of those may have appeared later than fyr. Hence, for the dataset used in this case this loop could be replaced with a simple check where only fyr is considered.
end
else
newlist=firmlist;
end
