% FIRMSINPC This function creates a list of firms extraxted according to caseflag.
% Input: first year fyr, last year lyr. Note the edgelists should be stored as 
% edgelistyr.tsv (where yr corresponds to the year)
% Explanation of the caseflag:
% caseflag=0; corresponds to taking the intersection of all giant components, 
% i.e., firms that have appear in the giant component each year between fyr and lyr 
% (this is what you get if caseflag is empty or not given).
% caseflag=1; corresponds to taking the union of all giant components of 
% different years.
% caseflag=2; corresponds to taking all firms.
% caseflag=3; all firms that are indirectly or directly connected to 
% the main component (the recursive selection used in the manuscript).
% OUTPUT: cell array of nameS of firms in the selection.
% REQUIREMENTS: elist2adj, giantComponentAlt
function firmlist=firmsinpc(fyr,lyr,caseflag)

firmlist={};

%Setting the flags.
%uflag: take either union or intersection of firms in different years, set to false if intersection
%rflag: take the main component or not, set to false if only PC is wanted

if nargin<=2 || isempty(caseflag) || caseflag==0
uflag=false;
rflag=false;
elseif caseflag==1 || caseflag==3
uflag=true;
rflag=false;
elseif caseflag==2
rflag=true;
end
 
%Loop through the year and update the list of firms found in the main components.
for yr=fyr:1:lyr
[firms1,individuals1,A1]=elist2adj(yr);%make an adjacency matrix from an edgelist
tempmat=spones(A1*A1');
Adj=tempmat-diag(diag(tempmat));
if rflag==false
[Tempmat,pcfirms]=giantComponentAlt(Adj);%find the firms in the giant component

if ~isempty(firmlist)
if uflag==true
[firmlist, IA, IB] = union(firms1(pcfirms),firmlist);
else
[firmlist, IA, IB] = intersect(firms1(pcfirms),firmlist);
end
else
firmlist=firms1(pcfirms);
end %isempty firmlist
else %rflag==true
[firmlist, IA, IB] = union(firms1,firmlist);
end
end

%In the case of 'recursive' selection the list of firms is extended.
if caseflag==3;

stopflag=false;

while stopflag==false
nf=length(firmlist);
for yr=fyr:1:lyr
[firms1,individuals1,A1]=elist2adj(yr);
[firmlistisect, IA, IB] = intersect(firms1,firmlist);
tempmat=A1*A1';
firmlist=union(firms1(find(sum(tempmat(IA,:),1))),firmlist);
end
if nf==length(firmlist)
stopflag=true;
end
end%end of while
end %end of if
