%This example demonstrates fitting the exponential search model to data.
%A sample of descriptive statistics (corresponding to years 2012-2015) is
%saved in demo_descriptives. The model is fitted to the corresponding 
%empirical distributions.

%Loading the data and the required packages.
%If you have extracted the packege to folder X/expsearch, 
%set mypath='/X/expsearch/' and run
addpath(genpath(mypath));
pkg load parallel; %making sure that parallel toolbox is in use
load demo2_descriptives; %this contains descriptives (a cell array) and 
%fimrlist (list of firms), fyr=2012, lyr=2015.

%Aggregating the yearly descriptive statistics.
aggdescriptives=aggregatedist(descriptives,true);

%Creating the initial adjacency matrix, i.e., the adjacency matrix correnponding 
%to year 2012 and the firms given in firmlist.
Ainit=initmatgc(fyr,descriptives{11,1},firmlist);

%Some parameters and the degree and distance distributions.
pn=aggdescriptives{9}; %probability of recruiting 'outsider'
p0=aggdescriptives{11}; %probability of opening vacancy
didisnew=aggdescriptives{2}; %distribution of distances of new individuals
dedisnew=aggdescriptives{4}; %distribution of degrees of new individuals
K=length(aggdescriptives{1}); 
%aggdescriptives{1} constains the degree distribution of all people
pout=aggdescriptives{10};

%Initial values used in the optimization. These are optional if you set bounds for the variables.
betap=0.6;
gammap=1.2;
dinf=8.1;

%Parameters of the simuilation and the optimization routine.
tolx=0.05; %tolerance for the termination
nsims=16; %number of simulations for one evaluation of the objective function
%Note 8 is rather small, it is recommended to have more simulations
ntime=lyr-fyr; %number of periods
cmax=12; %largest distance from which anyone is searched
%Note: parameters nsims, ntime, and cmax all affect how long it takes to evalute the objective function

%Finding the parameters, setting the upper and lower bounds. If you leave these empty, they will be generated in optimizer from the initial values betap, gammap, and dinf.
lbounds=[0 0.9 6];
ubounds=[7 2 10];
[outbeta,outgamma,outdinf,diagnostics]=optimizer(betap,gammap,dinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,tolx,lbounds,ubounds);
%Alternatively you may try without bounds
%[outbeta,outgamma,outdinf,diagnostics]=optimizer(betap,gammap,dinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,tolx);

%If you want you can try some of the Octave optimization routines and see if the result changes, e.g.
%f = @(params)simfitObj(params,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax);
%options2 = optimset('TolX', 0.001);
%[xopt,optval]=fminunc(f,[outbeta outgamma outdinf]',options2);
%Note that this is usually a good idea: the results improve.

%If you want you may try bootstrapping, but it is likely to take plenty of time!
%set nbs to the number of 'bootstraps'
%nbs=10;
%parameters=[p0 alpha K pn outbeta outgamma outdinf];
%[sep0,sepn,sebetap,segammap,sedinf,pns,p0s,betaps,gammaps,dinfs]=bootstrapper(parameters,pout,Ainit,nbs,ntime,nsims,cmax,tolx);

%If you want to get more thorough diagnostics than just the distance and 
%degree distributions of new individuals, run the following
%[Aouttemp,Afout,degdis,avrccoef,diagnostics,rundata]=simulator(Ainit,pn,pout,outbeta,outgamma,outdinf,K,ntime,[],[],[],nsims,cmax);

%Drawing pictures
figure(1);%distances of new individuals
clf; plot(aggdescriptives{2},'r'); hold on; plot(diagnostics{1}); axis([1 12 0 max(max(aggdescriptives{2}),max(diagnostics{1}))]);plot(12,aggdescriptives{2}(end),'r*'); plot(12,diagnostics{1}(end),'k*');
title('Distances of new individuals');

%degrees of new individuals
figure(2); 
clf; plot(aggdescriptives{4},'r'); hold on; plot(diagnostics{2}); axis([1 7 0 1]);
title('Degrees of new individuals');
