% FINDALLNEIGHBORS This function extracts people in the neighborhoods of firms.
% INPUTS:
% adjacency matrix A
% firms for which the search is done. Note that each position belongs to some 
% firm. Hence instead of giving positions we can input directly the firms to 
% which the positions belong,
% cmax is the maximum number of neighborhoods went through in the computation
% OUTPUTS:
% are cell arrays including one vector for each firm
% allpeople is the list of all people in the neighborhoods up to distance cmax,
% outpeople are people not in the aforementioned neighborhoods,
% dist contains the distances of people from the firm, degr their degrees,
% the indexing in dist corresponds to allpeople,
% degrees of outpeople are in degroutpeople,
% maxdist is the largest distance from which anyone is found (this is 
% outputted only for diagnostic purposes), 
% degroutpeople contains the degrees of outpeople
% REQUIREMENTS: fastsetdiff
function [allpeople,dist,degr,outpeople,degroutpeople,maxdist]=findallneighbors(A,firms,cmax)

if nargin<3 || isempty(cmax)
cmax=min(15,size(A,1)); %VERY IMPORTANT: maximum number of neighborhoods that is went through
else
cmax=min(cmax,size(A,1));
end

B=spones(A*A');
nof=length(firms);
maxpeople=zeros(nof,1);
maxdist=zeros(nof,1); %largest distance

for j=1:nof
idvs{j}=find(A(firms(j),:)~=0); %individuals of firm
allpeople{j}=[];
outpeople{j}=fastsetdiff([1:1:size(A,2)],idvs{j}); %people not in the board of a firm
maxpeople(j)=length(outpeople{j});
dist{j}=[];
degr{j}=[];
end %end of for

nofaux=nof;
countaux=[1:1:nof];
firmsaux=firms;
stopflag=false;
tempmat=B;
cntr=1;

while stopflag==false %this loop finds out distances
remind=[];
for i=1:nofaux %FOR1, loop through firms for which there are still some people left for whom the distance is not yet found
tempfirms=fastsetdiff(find(tempmat(firms(countaux(i)),:)~=0),firms(countaux(i)));%neighboring firms of firms(countaux(i))
trial=fastsetdiff(find(sum(A(tempfirms,:),1)),idvs{countaux(i)});%individuals affiliated to tempfirms that are not affiliated with fimr(countaux(i))

if isempty(trial) %all relevant neighnorhoods of firm firms(countaux(i)) have been went through
remind=[remind i];
else
trial=fastsetdiff(trial,allpeople{countaux(i)}); %find those people who are not already found

if isempty(trial)
remind=[remind i];
else
[allpeople{countaux(i)},sortind]=sort([allpeople{countaux(i)} trial]);%allpeople is the list of all individuals in neighborhoods of order less than cntr

tempdist=[dist{countaux(i)} cntr.*ones(1,length(trial))];
dist{countaux(i)}=tempdist(sortind);
maxdist(countaux(i))=cntr;

tempdegr=[degr{countaux(i)} sum(A(:,trial),1)];
degr{countaux(i)}=tempdegr(sortind);
end%end of if for inner isempty(trial) 
end %end of if for outer isempty(trial)
end%end of for loop over firms FOR1

countaux(remind)=[]; %update the list of firms that need to be went through
if isempty(countaux)
stopflag=true;
else
nofaux=length(countaux);
end

cntr=cntr+1;

if cntr>=cmax
stopflag=true;
end

if rem(cntr,4)==0 %just to avoid call to spones every period, without running spones tempmat would explode quite rapidly
tempmat=spones(B*tempmat);
else
tempmat=B*tempmat;
end
end %end of while loop1
clear B; clear tempmat; clear countaux;

cntr=1;
stopflag=false;

%Loop2, to find outpeople
while stopflag==false && cntr<=nof
if length(allpeople{cntr})==maxpeople(cntr) %this is to avoid unnecessary call to fastsetdiff
outpeople{cntr}=[];
degroutpeople{cntr}=[];
else
outpeople{cntr}=fastsetdiff(outpeople{cntr},allpeople{cntr});
degroutpeople{cntr}=sum(A(:,outpeople{cntr}));
end
cntr=cntr+1;
end %end of while loop2
