% Compute the clustering coefficient per node.
% Ci = the average local clustering, where 
% Ci = (number of triangles connected to i) / (number of triples centered on i)
% Ref: M. E. J. Newman, "The structure and function of complex networks"
% Note: Valid for directed and undirected graphs
%
% INPUT: adjacency matrix, nxn
% OUTPUT: the average clustering coefficient (aveC) and the
%         clustering coefficient vector C per node (where mean(C) = aveC)
%
% This function is created by modifying clustCoeff of the network toolbox.

function [aveC,C] = clustCoeffAlt(adj)

n = length(adj);
adj = adj>0;  % no multiple edges

S = true; %S=true if the graph is directed
if adj==transpose(adj); S = false; end
if S; coeff=1; end

if S==true
  deg = sum(adj) + sum(adj'); % total degree
else   % undirected graph: indeg=outdeg
  deg = sum(adj) + diag(adj)';  % add self-loops twice, if any
end

C=zeros(n,1); % initialize clustering coefficient

% multiplication change in the clust coeff formula
% to reflect the edge count for directed/undirected graphs
coeff = 2;
if S==true; coeff=1; end

for i=1:n
  
  if deg(i)==1 || deg(i)==0; C(i)=0; continue; end

  neigh=find(adj(i,:)>0);
temp=adj(neigh,neigh);
sl=sum(diag(temp)); % counting the number of self-loops

if S==false && sl==0    % undirected simple graph
    edges_s=sum(sum(temp))/2;
    
elseif S==false && sl>0
    edges_s=(sum(sum(temp))-sl)/2+sl; % counting the self-loops only once

elseif S==true   % directed graph (not necessarily simple)
    edges_s=sum(sum(temp));
end

%  neigh2=kneighbors(adj,i,1);
%  edges_s2=numEdges(subgraph(adj,neigh2));
%if length(neigh2)!=length(neigh)
%0
%end

  C(i)=coeff*edges_s/(deg(i)*(deg(i)-1));

end

aveC=sum(C)/n;
