% Algorithm for finding connected components in a graph
% INPUTS:
% Adj is the adjacency matrix of the graph.
% origAdj is the original adjacency matrix 
% (in case the questions is of a a subgraph)
% indices: are the "name" indices of the nodes corresponding to adj
% names are the names of these indices in the original indexing 
% corresponding to origAdj 
% adj is the adjacency matrix correspoding to the subgraph 
% composed of nodes in names
% Note: Valid for undirected graphs only.
% OUTPUTS: a list of the components comp_names{i}=[j1,j2,...jk] 
% indexing corresponding to origAdj, nnodes contains the number of nodes 
% in each component
%
% Other routines used: fastsetdiff, findConnCompIAlt.m
function [comp_names,nnodes] = findConnCompAlt(origAdj,adj,names)

if nargin<2
adj=origAdj;
names=[1:1:length(adj)];
end

deg=sum(adj);% degrees
[deg,indc]=sort(deg,'descend'); %this trick is only because it is likely to improve the performance of the below loop
adj=adj(indc,indc);
names=names(indc);
comp_names={};% initialize components matrix
ntot=length(deg);
ncount=0;
gcount=0;
nnodes=[];
n=size(adj,1);
tempvec=1:1:length(origAdj);
togolist=1:1:length(adj);

while ~isempty(togolist)

i=togolist(1);
    if deg(i)>0 %avoid extra effort when there are no outgoing links at all
%tic ();
        comp=findConnCompIAlt(adj,i);%This is the step that takes some time, if you can improve this please do so
%toc
	gcount=gcount+1;
	nnodes(gcount)=length(comp);
	ncount=ncount+nnodes(gcount);
	comp_names{gcount}=names(comp);
	if ncount==ntot %avoid going to fastsetdiff if unnecessary
	break;
	else
        togolist=fastsetdiff(togolist,comp);
	end
    elseif deg(i)==0
        ncount=ncount+1;
	gcount=gcount+1;
	nnodes(gcount)=1;
	comp_names{gcount}=names(i);
	if ncount==ntot %avoid going to fastsetdiff if unnecessary
	break;
	else
	togolist=fastsetdiff(togolist,i);
	end
    end
end
