% Find the connected component to which node "i" belongs to
%
% INPUTS: adjacency matrix and index of the key node
% OUTPUTS: all node indices of the nodes in the same group 
%          to which "i" belongs to (including "i")
%
% Note: Only works for undirected graphs.
% Modified from the function findConnCompI in network toolbox 

function comp=findConnCompIAlt(adj,i)
n=length(adj);
adj=adj+speye(n,n); %add self loops so that adj becomes primitive
neigh=find(adj(i,:)>0);
neigh=unique([neigh i]);    % add i to its own component
counter=1;
tempmat=adj;
%What is a bit puzzling here is that matrix product is done over and over again,
%reversing the looping would be better
%first loop through matrix products, then through indices (just an idea)
%perhaps some kind of vectorization: for each node a row vector indicating which of them belong to same component
%but at least this is better than the original findConnCompI
while 1
  len0=length(neigh);
  tempmat=spones(adj*tempmat);
  neigh=unique([neigh find(tempmat(i,:)>0)]);
  if len0==length(neigh)  % if no new neighbors are found, return component
    comp=neigh;
    return
  end
counter=counter+1;
end

end  % end of function
