% INITMATGC This function creates the initial matrix of year yr corresponding to
% the selection of firms given by firmlist, and adjust the number of seats to 
% fit ms (see below).
% INPUTS:
% yr = year
% ms is an m-dimensional vector of (m=number of firms) containing the number of 
% seats for each firm.
% firmlist is the list of firms
% pcflag (optional) can be set true if only companies in the main component are 
% to be considered.
% OUTPUT: 
% adjacency matrix where the number of seats has been adjusted according to ms.
% REQUIREMENTS: elist2adj, fastsetdiff, findConnCompAlt, findConnCompIAlt, 
% gianComponentAlt
function output=initmatgc(yr,ms,firmlist,pcflag)

[firms1,individuals1,A1]=elist2adj(yr);%make adjacency matrices from an edgelist
if isempty(firmlist)
if nargin<=3 || pcflag==true
[GC,gc_nodes]=giantComponentAlt(A1*A1');
A1temp=A1(gc_nodes,:);
output=A1temp(:,any(A1temp,1));
else
output=A1;
end

else
[auxisect,indA,indB]=intersect(firms1,firmlist);
A1temp=A1(indA,:);
output=A1temp(:,any(A1temp,1));
end

%Appending the matrix according to the ms
if ~isempty(ms)
nf=size(output,1);%number of firms
for j=1:nf;
dif=sum(output(j,:))-ms(j);
if dif<0 %add more board members
auxmat=zeros(nf,-dif);auxmat(j,:)=1;
output=[output auxmat];
elseif dif>0 %remove some board members
ind=find(output(j,:)==1);
output(j,ind(1:dif))=0;
end
end
indic=any(output);
output(:,!indic)=[];
end
