% OPENVACANCIES generates open vacancies given the parameters p0 and alpha. 
% INPUTS: 
% adjacency matrix A of a two-mode network (firm,s individuals), 
% model parameters p0 and alpha 
% rstate is the state of the random number generator
% OUTPUTS: 
% co is a cell array in which co{j} is the list of individuals in firm j 
% who have lost a position in the firm,
% nov is the vector containing the number of vacancies of a firm 
% nov(j) being the number of vacancies for firm j
% degolds is vector of degrees of people who have left their positions
% Warning: MATRIX A should be a zero-one matrix, otherwise the function fails
% REQUIREMENTS: fastsetdiff
function [co,nov,degolds,rstate]=openvacancies(A,pout,rstate)

if nargin==3 && ~isempty(rstate)
rand('state',rstate);
end

[nfirm,nidv]=size(A);
degidv=sum(A,1);
firms=[];
nov=zeros(nfirm,1);%vector of number of new vacancies for the firms
degolds=[];
for k=1:nidv%loop over all individuals, loop A
if degidv(k)>length(pout)
degidv(k)=length(pout);
end
trial=(rand(degidv(k),1)<=pout(degidv(k)));%vacancies held by an individual that are opened
if sum(trial)>=1 && ~isempty(trial); %if at least one vacancy is opened, if A
temp = A(:,k);
if length(temp(temp==1))==length(trial) %if B
temp(temp==1)=trial;%what is going on? trial is vector of zeros and ones, this vector is set to temp (why) trial==1 means that vacancy is opened, now temp contains one in every company from which the individual leaves 
degolds=[degolds degidv(k).*[1:1:sum(trial)]];%number of vacancies opened is sum(trial), degree of individual is degidv(k) 
rtrial=find(temp==1);%companies for which vacancies open
%cotrial=k.*ones(size(rtrial)); 

[oldro,ia,ib]=intersect(rtrial,firms);%find firms for which already some vacancies open
[newro,ind]=fastsetdiff(rtrial',oldro');%find firms not previously listed for which vacancies are opened
if ~isempty(newro) %if there are new firms
firms=[firms; newro'];%update firm list
co(newro)={k}; %set individual k to the list of people correponding to newfirms
nov(newro)=nov(newro)+1; %number of open vacancies for new firms is updated
end

if ~isempty(oldro)%, if C
nov(oldro)=nov(oldro)+1; %number of open vacancies for old firms is updated
for j=1:length(oldro) %update the list of individuals of previously listed firms
co{oldro(j)}=[co{oldro(j)} k];
end%enf of for
end%end of if C
end%end of if B
end%end of if A
end%end of for A

if isempty(firms)
co=[];
nov=[];
end

rstate=rand('state',rstate);
