% OPTIMIZER This is a very simple optimization algorithm based on 
% a coordinate search and the golden section line search method. 
% One purpose is to avoid unnecessary function evaluations that are constly. 
% The idea is to optimize the objective function; square error of degree 
% and distance distributions, as if the different parameters affect the 
% different components independently: betap affects the degree dsitribution, 
% gammap affects the distance distribution except for the last component 
% (outsiders) which is affected by dinf. If you want the optimizer not to 
% change some of the parameters set the upper and lower bounds of those 
% parameters tight enough.
% rstate is the state of the random number generator
% INPUTS:
% the usual parameters and tolx (the tolerance for the termination); 
% the iteration is terminated when all the parameters change less than tolx
% lower and upper bounds of variables are optional, lbounds and ubounds are 
% vectors corresponding to the variables betap, gammap, and dinf. 
% If the bounds are given, the initial values can be left empty.
% OUTPUTS:
% optimal parameter values and a cell array diagnosotics containing 
% the distance and degree distributions of new people
% initial state of the random number generator is outputted (=rstate if
% rstate is given as an input)
% REQUIREMENTS: simobj.m (objective function), simuloop.m 
% (simulation of the model) + the m-files required within simuloop 
% (check the help of simuloop for details).
function [outbeta,outgamma,outdinf,diagnostics,rstateinit]=optimizer(betap,gammap,dinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,tolx,lbounds,ubounds,rstate)

%Initializations
maxiter=5;%this is obsolete, but can be set for diagnostic purposes
%The maximum number of iterations is determined by 1/gr, 
%maxnumber n is given by (1/gr)^(n+1)(length of initial interval)<=tolx
if nargin<16 || isempty(rstate)
rstateinit=rand('state');
else
rstateinit=rstate;
end

gr=(sqrt(5)+1)/2;%Golden ratio
stopflag1=false;
stopflag2=false;
stopflag3=false;
stopflag=false;

%Upper and lower bounds of the variables
if nargin<15 || isempty(lbounds)
ub1=2*betap;
lb1=0;
ub2=2*gammap;
lb2=0;
ub3=2*dinf;
lb3=0.5*dinf;
else
ub1=ubounds(1);
lb1=lbounds(1);
ub2=ubounds(2);
lb2=lbounds(2);
ub3=ubounds(3);
lb3=lbounds(3);
end

%Trial points
tr1beta=ub1-(ub1-lb1)/gr;
tr2beta=lb1+(ub1-lb1)/gr;
tr1gam=ub2-(ub2-lb2)/gr;
tr2gam=lb2+(ub2-lb2)/gr;
tr1dinf=ub3-(ub3-lb3)/gr;
tr2dinf=lb3+(ub3-lb3)/gr;

%Evaluating the objective function at the trial points
[termbetap1,termgammap1,termdinf1,dis1,dis2]=simobj(tr1beta,tr1gam,tr1dinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,rstate);

[termbetap2,termgammap2,termdinf2]=simobj(tr2beta,tr2gam,tr2dinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,rstate);

trial1=(tr1beta+tr2beta)/2; %these are set just in case the trial points are 'too close'
trial2=(tr1gam+tr2gam)/2;
trial3=(tr1dinf+tr2dinf)/2;
outbeta=trial1;
outgamma=trial2;
outdinf=trial3;

cntr=1;%iteration counter (just in case you have set maxiter)

%The optimization loop
while stopflag==false

if stopflag1==false
if abs(tr1beta-tr2beta)>tolx
if termbetap1<termbetap2
ub1=tr2beta;
tr2beta=tr1beta;
termbetap2=termbetap1;
termbetap1=[];
trial1=ub1-(ub1-lb1)/gr;
outbeta=(trial1+tr2beta)/2;
else
lb1=tr1beta;
tr1beta=tr2beta;
termbetap1=termbetap2;
termbetap2=[];
trial1=lb1+(ub1-lb1)/gr;
outbeta=(trial1+tr1beta)/2;
end
else
stopflag1=true;
trial1=outbeta;
end
end

if stopflag2==false
if abs(tr1gam-tr2gam)>tolx
if termgammap1<termgammap2
ub2=tr2gam;
tr2gam=tr1gam;
termgammap2=termgammap1;
termgammap1=[];
trial2=ub2-(ub2-lb2)/gr;
outgamma=(trial2+tr2gam)/2;
else
lb2=tr1gam;
tr1gam=tr2gam;
termgammap1=termgammap2;
termgammap2=[];
trial2=lb2+(ub2-lb2)/gr;
outgamma=(trial2+tr1gam)/2;
end
else
stopflag2=true;
trial2=outgamma;
end
end

if stopflag3==false
if abs(tr1dinf-tr2dinf)>tolx
if termdinf1<termdinf2
ub3=tr2dinf;
tr2dinf=tr1dinf;
termdinf2=termdinf1;
termdinf1=[];
trial3=ub3-(ub3-lb3)/gr;
outdinf=(trial3+tr2dinf)/2;
else
lb3=tr1dinf;
tr1dinf=tr2dinf;
termdinf1=termdinf2;
termdinf2=[];
trial3=lb3+(ub3-lb3)/gr;
outdinf=(trial3+tr1dinf)/2;
end
else
stopflag3=true;
trial3=outdinf;
end
end


[test1,test2,test3,dis1,dis2]=simobj(trial1,trial2,trial3,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,rstate);

if isempty(termbetap1)
termbetap1=test1;
tr1beta=trial1;
else
termbetap2=test1;
tr2beta=trial1;
end

if isempty(termgammap1)
termgammap1=test2;
tr1gam=trial2;
else
termgammap2=test2;
tr2gam=trial2;
end

if isempty(termdinf1)
termdinf1=test3;
tr1dinf=trial3;
else
termdinf2=test3;
tr2dinf=trial3;
end
cntr=cntr+1;
if (stopflag1 & stopflag2 & stopflag3) || cntr>=maxiter
stopflag=true;
end
end %End of main loop

%Diagnostics; distance and degree distributions of new people
diagnostics{1}=dis1;
diagnostics{2}=dis2;
