% SIMFITOBJ Alternative objective function for fitting the parameters of 
% the exponential search model.
% INPUTS:
% params contains the three parameters (beta, gamma, and dinf) 
% that are to be fitted
% parameters pn, p0, alpha, K 
% didisnew is the empirically observed distance distribution of newly 
% appointed people
% dedisnew is the empirically observed degree distribution of newly 
% appointed people 
% Ainit is the intial adjacency matrix
% nsim is the number of times the simulation is run
% ntime is the number of periods used in the simulations
% cmax is the maximum number of neighborhoods that is went though in the 
% computation of distances (default is set in parsefitoptions)
% seed is the vector of seed value of the random number generator, seed should
% contain the same number of elements as there are simulations, i.e., nsim
% rstate is the state of the random number generator
% OUTPUT:
% value of the objective function
% REQUIREMENTS: add2dist, fastsetdiff, findallneighbors, newrandpeople, 
% simuloop, updateadj
function outval=simfitObj(params,pn,pout,K,didisnew,dedisnew,Ainit,nsim,ntime,cmax,rstate)

if isempty(nsim)
nsim=1;
end

%parameters of the model 
betap=params(1); 
gammap=params(2); 
dinf=params(3);

%inititializations
distd=[];degnewd=[];

%MAIN 'LOOP'
%run the simulation nsim times and take the average
%this part is parallelized!
[distd,degnewd]=pararrayfun(nproc,@(seed) simuloop(Ainit,pn,pout,betap,gammap,dinf,K,ntime,cmax,rstate,seed),[0:1:nsim-1],"Uniformoutput",false,"VerboseLevel",0);%the seed runs from 1 to nsim

totaldis=[];
totaldeg=[];

%Taking the averages
for j=1:nsim 
totaldis=add2dist(distd{j},totaldis,1);
totaldeg=add2dist(degnewd{j},totaldeg,1);
end

%the relevant distributions
dis1=totaldis(2:end)./sum(totaldis(2:end));%distd; dist zero means that not found
dis2=totaldeg(2:end)./sum(totaldeg(2:end));

nmin1=min(length(dis1),length(didisnew));
nmin2=min(length(dis2),length(dedisnew));

outval=norm(dis1(1:nmin1)-didisnew(1:nmin1))^2+norm(dis2(1:nmin2)-dedisnew(1:nmin2))^2;

if length(dis1)>length(didisnew)
outval=outval+norm(dis1(nmin1+1:end))^2;
elseif length(dis1)<length(didisnew)
outval=outval+norm(didisnew(nmin1+1:end))^2;
end

if length(dis2)>length(dedisnew)
outval=outval+norm(dis2(nmin2+1:end))^2;
elseif length(dis1)<length(dedisnew)
outval=outval+norm(dedisnew(nmin2+1:end))^2;
end

outval=100*outval;
