% UPDATEADJ Updating a two-mode network when some edges are rewired, i.e., some
% people are changed.
% INPUTS:
% A is the initial adjacency matrix
% model parameters alpha, betap, gammap, p0, pn, K, dinf
% cmax is the upper bound for how many neighborhoods are went through, 
% when computing the distances
% rstate is the state of the random number generator
% OUTPUTS:
% Anew is the new adjacency matrix, 
% distout is the distance distribution of new people,
% degnewout is the degree distribution of new people,
% degoldout is the degree distribution of people who lost a position
% REQUIREMENTS:: fastsetdiff, findallneighbors, newrandpeople, openvacancies
function [Anew,distout,degnewout,degoldout,rstate]=updateadj(A,betap,gammap,pout,pn,K,dinf,cmax,rstate)

if nargin<8
cmax=[];
end

[cols,novs,degold,rstate]=openvacancies(A,pout,rstate);%generating open vacancies
firms=find(novs>=1);%find firms for which at least one vacancy is opened
firms1=find(novs==1);%find firms for which exactly one vacancy is opened, firms in which multiple vacacies open are treated separately

if ~isempty(firms)
[allpeople,dist,degr,outpeople,degroutpeople,maxdist]=findallneighbors(A,firms,cmax);%finding people within different distances
indx=lookup(firms,firms1,'m');%this takes intersection
indx(indx==0)=[]; %note indx==0 if fimrs1 has an index less than the least index

%filling the vacancies in firms in which only one vacancy has opened
if ~isempty(indx)
[index,distnew,degnew,Anew,rstate]=newrandpeople(A,firms1,cols(firms1),[],allpeople(indx),dist(indx),degr(indx),outpeople(indx),degroutpeople(indx),betap,gammap,pn,K,dinf,rstate);
else
Anew=A;
distnew=[];
degnew=[];
end

%firms for which multiple vacancies have opened
[firms2,indx]=fastsetdiff(firms,firms1);
r=length(firms2);
distnewtemp=[];
degnewtemp=[];
for k=1:r

idvs=cols{firms2(k)};
nn=length(idvs);
rand('state',rstate);
indices=randperm(nn);
rstate=rand('state');
nolist=[]; %the idea is to keep track of people who get a position in a company and to take care that no person gets appointed multiple 
%times to the same firm. For small networks it may even happen that some times there is no one left to be appointed. Here (see newrandpeople) it is assumed that in such cases the new person is an outsider. Consequently, for small networks (#firms<10, #positions<20) the estimate of pn can be  slightly biased.

for j=1:nn
idv=idvs(indices(j));
tempcell={idv};
[index,dst,dgrnew,Anew,rstate]=newrandpeople(Anew,firms2(k),tempcell,nolist,allpeople(indx(k)),dist(indx(k)),degr(indx(k)),outpeople(indx(k)),degroutpeople(indx(k)),betap,gammap,pn,K,dinf,rstate);

distnewtemp=[distnewtemp dst];
degnewtemp=[degnewtemp dgrnew];
if index~=0 %note index==0 if the new person is an outsider
nolist=[nolist index];
end
end%end of for
end%end of for

distnew=[distnew; distnewtemp'];%distances of new people
degnew=[degnew; degnewtemp']; %degrees of new people
nnew=size(Anew,2)-size(A,2); %number of completely new people
Anew(:,!any(Anew))=[]; %remove columns corresponding to people who no loner have any position

if ~isempty(distnew)
distout=histc(distnew,[0:1:max(distnew)]);%distribution of distances from which the new recruits are taken conditional on they are from the existing network, note distnew=m if not found in neihgborhoods and zero if complete outsider, i.e., nnew==distout(1)
degnewout=histc(degnew,[0:1:max(degnew)]);%distribution of degrees of the new recruits, zero if new
degoldout=histc(degold,[1:1:max(degold)]);%distribution of degrees of people who lost a positions
else
distout=[];
degnewout=[];
degoldout=[];
end
if size(distout,2)==1
distout=distout';
end
if size(degnewout,2)==1
degnewout=degnewout';
end
if size(degoldout,2)==1
degoldout=degoldout';
end

else
Anew=A;
distout=[];
degnewout=[];
degoldout=[];
end
rstate=rand('state');
